package ink.wgink.module.activiti.service.activiti.impl;

import com.alibaba.fastjson.JSONObject;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.SearchException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.module.activiti.service.activiti.IActivitiModelService;
import ink.wgink.util.ResourceUtil;
import org.activiti.editor.constants.ModelDataJsonConstants;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.repository.Model;
import org.apache.batik.transcoder.TranscoderInput;
import org.apache.batik.transcoder.TranscoderOutput;
import org.apache.batik.transcoder.image.PNGTranscoder;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.*;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: ActivitiModelServiceImpl
 * @Description: 流程模型
 * @Author: WangGeng
 * @Date: 2021/7/12 22:01
 * @Version: 1.0
 **/
@Service
public class ActivitiModelServiceImpl extends DefaultBaseService implements IActivitiModelService, ModelDataJsonConstants {

    @Autowired
    private RepositoryService repositoryService;

    @Override
    public void saveModel(String modelId, String name, String description, String json_xml, String svg_xml) throws Exception {
        Model model = repositoryService.getModel(modelId);
        if (model == null) {
            throw new SearchException("模型不存在");
        }
        JSONObject modelObject = JSONObject.parseObject(model.getMetaInfo());

        modelObject.put(MODEL_NAME, name);
        modelObject.put(MODEL_DESCRIPTION, description);
        model.setMetaInfo(modelObject.toString());
        model.setName(name);
        repositoryService.saveModel(model);
        repositoryService.addModelEditorSource(model.getId(), json_xml.getBytes(ISystemConstant.CHARSET_UTF8));

        InputStream svgStream = new ByteArrayInputStream(svg_xml.getBytes(ISystemConstant.CHARSET_UTF8));
        TranscoderInput input = new TranscoderInput(svgStream);

        PNGTranscoder transcoder = new PNGTranscoder();
        // Setup output
        ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        TranscoderOutput output = new TranscoderOutput(outStream);

        // Do the transformation
        transcoder.transcode(input, output);
        final byte[] result = outStream.toByteArray();
        repositoryService.addModelEditorSourceExtra(model.getId(), result);
        outStream.close();

    }

    @Override
    public JSONObject getEditorJson(String modelId) throws UnsupportedEncodingException {
        Model model = repositoryService.getModel(modelId);
        if (model == null) {
            throw new SearchException("模型不存在");
        }
        JSONObject result;
        if (StringUtils.isNotEmpty(model.getMetaInfo())) {
            result = JSONObject.parseObject(model.getMetaInfo());
        } else {
            result = new JSONObject();
            result.put(MODEL_NAME, model.getName());
        }
        result.put(MODEL_ID, model.getId());
        String readTree = new String(repositoryService.getModelEditorSource(model.getId()), ISystemConstant.CHARSET_UTF8);
        JSONObject editorJson = JSONObject.parseObject(readTree);
        result.put("model", editorJson);
        return result;
    }

    @Override
    public JSONObject getStencilset() throws IOException {
        FileInputStream fileInputStream = new FileInputStream(ResourceUtil.getResourceFile("static/stencilset.json"));
        String result = IOUtils.toString(fileInputStream, ISystemConstant.CHARSET_UTF8);
        return JSONObject.parseObject(result);
    }
}
